﻿using System.IdentityModel.Tokens.Jwt;
using System.Reflection;
using Exceptionless;
using FluentValidation;
using FluentValidation.AspNetCore;
using Hotline.Api.Realtimes;
using Hotline.Application;
using Hotline.Application.Contracts;
using Hotline.Application.Contracts.Configurations;
using Hotline.CacheManager;
using Hotline.CallCenter.Devices;
using Hotline.Identity.Accounts;
using Hotline.NewRock;
using Hotline.Permissions;
using Hotline.Repository.SqlSugar;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Settings;
using Mapster;
using MapsterMapper;
using MediatR;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Identity;
using Microsoft.IdentityModel.Tokens;
using Microsoft.OpenApi.Models;
using Serilog;
using XF.Domain.Dependency;
using XF.Domain.Filters;
using XF.Domain.Options;
using XF.Domain.Password;

namespace Hotline.Api;

internal static class StartupExtensions
{
    const string CorsOrigins = "CorsOrigins";
    internal static WebApplication ConfigureServices(this WebApplicationBuilder builder)
    {
        var services = builder.Services;
        var configuration = builder.Configuration;

        services.AddHttpContextAccessor();

#if DEBUG
        builder.WebHost.UseUrls("http://*:50110");
#endif

        services.Configure<DeviceConfigs>(d => configuration.GetSection(nameof(DeviceConfigs)).Bind(d));
        services.Configure<IdentityConfiguration>(d => configuration.GetSection(nameof(IdentityConfiguration)).Bind(d));

        // Add services to the container.
        services
            .BatchInjectServices()
            .AddApplication()
            .AddScoped<IPasswordHasher<Account>, PasswordHasher<Account>>()
            ;

        //Authentication
        services.RegisterAuthentication(configuration);

        services.AddControllers(options =>
        {
            options.Filters.Add<UnifyResponseFilter>();
            options.Filters.Add<UserFriendlyExceptionFilter>();
        })
            //            .AddDapr(d =>
            //            {
            //#if DEBUG
            //                d.UseHttpEndpoint("http://192.168.100.223:50112");
            //#endif
            //            })
            ;

        // Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
        services.AddEndpointsApiExplorer();

        //swagger
        services.RegisterSwagger();

        //signalR
        services.RegisterSignalR(configuration);

        /* CORS */
        services.RegisterCors(configuration, CorsOrigins);

        //mapster
        services.RegisterMapper();

        //mediatr
        services.AddMediatR(Assembly.GetExecutingAssembly(), typeof(ApplicationStartupExtensions).Assembly);

        //迅时IPPBX
        var deviceConfigs = configuration.GetSection("DeviceConfigs").Get<DeviceConfigs>();
        services.AddNewRock(deviceConfigs.Address);

        //sqlsugar
        services.AddSqlSugar(configuration, "Hotline");

        //cache
        services.AddCache(d =>
            {
                d.ConnectionString = configuration.GetConnectionString("Redis");
                d.Prefix = "Hotline";
            });

        //validator
        services.AddFluentValidationAutoValidation(config =>
        {
            config.DisableDataAnnotationsValidation = true;
        })
            .AddValidatorsFromAssembly(typeof(AppContractsStartupExtensions).Assembly);

        //ex log
        services.AddExceptionless(configuration);

        services.AddSingleton<IAuthorizationPolicyProvider, AuthorizationPolicyProvider>();
        services.AddSingleton<IAuthorizationHandler, PermissionHandler>();

        return builder.Build();
    }

    internal static WebApplication ConfigurePipelines(this WebApplication app)
    {
        app.UseSerilogRequestLogging();

        var useExless = app.Configuration.GetSection("Exceptionless:InUse").Get<bool>();
        if (useExless)
        {
            app.UseExceptionless();
        }

        var swaggerEnable = app.Configuration.GetSection("Swagger").Get<bool>();
        // Configure the HTTP request pipeline.
        if (swaggerEnable)
        {
            app.UseSwagger();
            app.UseSwaggerUI();
        }

        app.UseCors(CorsOrigins);

        app.UseAuthentication();
        app.UseAuthorization();
        app.MapHub<CallCenterHub>("/hubs/callcenter");

        //app.UseCloudEvents();
        app.MapControllers()
            .RequireAuthorization();
        //app.MapSubscribeHandler();

        return app;
    }
}