﻿using XF.Domain.Cache;
using XF.Domain.Dependency;

namespace Hotline.CallCenter.BlackLists
{
    public interface IBlacklistDomainService
    {
        /// <summary>
        /// 将电话加入黑名单
        /// </summary>
        Task AddAsync(Blacklist blacklist, CancellationToken cancellationToken = default);

        /// <summary>
        /// 删除黑名单信息
        /// </summary>
        /// <param name="phoneNo"></param>
        Task Remove(string phoneNo);

        /// <summary>
        /// 是否在黑名单中
        /// </summary>
        /// <param name="phoneNo"></param>
        /// <returns></returns>
        bool IsInBlacklist(string phoneNo);
    }

    public class BlacklistDomainService : IBlacklistDomainService, IScopeDependency
    {
        private readonly ITypedCache<Blacklist> _blackCache;
        private readonly IBlacklistRepository _blacklistRepository;

        public BlacklistDomainService(ITypedCache<Blacklist> blackCache, IBlacklistRepository blacklistRepository)
        {
            _blackCache = blackCache;
            _blacklistRepository = blacklistRepository;
        }

        /// <summary>
        /// 将电话加入黑名单
        /// </summary>
        public async Task AddAsync(Blacklist blacklist, CancellationToken cancellationToken = default)
        {
            blacklist.Id = await _blacklistRepository.AddAsync(blacklist, cancellationToken);
            _blackCache.GetOrAdd(Blacklist.GetKey(blacklist.PhoneNo), blacklist, ExpireMode.Absolute, TimeSpan.FromSeconds(blacklist.Duration));
        }

        /// <summary>
        /// 删除黑名单信息
        /// </summary>
        /// <param name="phoneNo"></param>
        public async Task Remove(string phoneNo)
        {
            _blackCache.Remove(Blacklist.GetKey(phoneNo));
            await _blacklistRepository.RemoveAsync(d => d.PhoneNo == phoneNo, true);
        }

        /// <summary>
        /// 是否在黑名单中
        /// </summary>
        /// <param name="phoneNo"></param>
        /// <returns></returns>
        public bool IsInBlacklist(string phoneNo)
        {
            var blacklist = _blackCache.Get(Blacklist.GetKey(phoneNo));
            return blacklist != null;
        }
    }
}
