﻿using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Enums.FlowEngine;
using MapsterMapper;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Hotline.FlowEngine.Definitions;

public class DefinitionDomainService : IDefinitionDomainService, IScopeDependency
{
    private readonly IDefinitionRepository _definitionRepository;
    private readonly IMapper _mapper;

    public DefinitionDomainService(IDefinitionRepository definitionRepository, IMapper mapper)
    {
        _definitionRepository = definitionRepository;
        _mapper = mapper;
    }

    public async Task<string> AddAsync(AddDefinitionDto dto, CancellationToken cancellationToken)
    {
        var definition = _mapper.Map<Definition>(dto);
        return await _definitionRepository.AddAsync(definition, cancellationToken);
    }

    /// <summary>
    /// 发布流程模板
    /// </summary>
    /// <param name="id"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    public async Task PublishAsync(string id, CancellationToken cancellationToken)
    {
        var definitionTemp = await _definitionRepository.GetAsync(id, cancellationToken);
        if (definitionTemp == null)
            throw UserFriendlyException.SameMessage("无效模板编号");
        if (definitionTemp.Status != EDefinitionStatus.Temporary)
            throw UserFriendlyException.SameMessage("非草稿模板不允许发布");

        ValidateDefinition(definitionTemp);

        var lastVersion = await GetLastVersionAsync(definitionTemp!.Code, cancellationToken);
        definitionTemp.Version = lastVersion + 1;
        definitionTemp.Status = EDefinitionStatus.Enable;

        await _definitionRepository.UpdateAsync(definitionTemp, cancellationToken);
    }

    /// <summary>
    /// 发布（保存并发布）
    /// </summary>
    public async Task PublishAsync(AddDefinitionDto dto, CancellationToken cancellationToken)
    {
        var id = await AddAsync(dto, cancellationToken);
        await PublishAsync(id, cancellationToken);
    }

    public async Task<int> GetLastVersionAsync(string code, CancellationToken cancellationToken)
    {
        var definition = await GetLastVersionDefinitionAsync(code, cancellationToken);
        return definition?.Version ?? 0;
    }

    public async Task<Definition?> GetLastVersionDefinitionAsync(string code, CancellationToken cancellationToken)
    {
        return await _definitionRepository.Queryable()
            .Where(d => d.Code == code)
            .OrderByDescending(d => d.Version)
            .Take(1)
            .FirstAsync();
    }

    public async Task<Definition?> GetLastVersionDefinitionByModuleCodeAsync(string moduleCode, CancellationToken cancellationToken)
    {
        return await _definitionRepository.Queryable()
            .Where(d => d.ModuleCode == moduleCode)
            .OrderByDescending(d => d.Version)
            .Take(1)
            .FirstAsync();
    }

    /// <summary>
    /// 未开启的流程，查找第2个节点的模板配置信息，用作开始流程的时候传参
    /// </summary>
    /// <param name="definitionCode"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    public async Task<IReadOnlyList<StepDefine>> GetSecondStepsAsync(string definitionCode, CancellationToken cancellationToken)
    {
        var definition = await GetLastVersionDefinitionAsync(definitionCode, cancellationToken);
        if (definition == null)
            throw new UserFriendlyException($"无效模板编码, code:{definitionCode}", "无效模板编码");
        return definition.FindSteps(definition.FindStartStep().NextSteps);
    }

    #region private
    private void ValidateDefinition(Definition? definition)
    {
        if (definition is null)
            throw new UserFriendlyException("无效 Definition Id");
        if (definition.Code == null)
            throw UserFriendlyException.SameMessage("无效Code");
        if (definition.Status is not EDefinitionStatus.Temporary)
            throw UserFriendlyException.SameMessage("只能发布草稿状态的模板");
        if (!definition.Steps.Any())
            throw UserFriendlyException.SameMessage("未配置流程节点");
        var startSteps = definition.Steps.Where(d => d.StepType == EStepType.Start).ToList();
        if (!startSteps.Any())
            throw UserFriendlyException.SameMessage("未配置开始节点");
        if (startSteps.Count > 1)
            throw UserFriendlyException.SameMessage("开始节点只能有一个");

        var endSteps = definition.Steps.Where(d => d.StepType == EStepType.End).ToList();
        if (!endSteps.Any())
            throw UserFriendlyException.SameMessage("未配置结束节点");
        if (endSteps.Count > 1)
            throw UserFriendlyException.SameMessage("结束节点只能有一个");

        foreach (var step in definition.Steps)
        {
            if (step.StepType != EStepType.End && !step.NextSteps.Any())
                throw UserFriendlyException.SameMessage($"{step.Code}节点未配置下一节点");
        }

        if (definition.Steps.GroupBy(d => d.Code).Count() < definition.Steps.Count)
            throw UserFriendlyException.SameMessage("一个模板内不允许出现code相同的节点");

        if (definition.Steps.Count(d => d.NextSteps.Any(x => x.Code == "End")) > 1)
            throw UserFriendlyException.SameMessage("结束节点只能有一个上级节点");
    }

    #endregion
}