﻿using Hotline.FlowEngine;
using Hotline.Share.Enums.Order;
using XF.Domain.Authentications;
using XF.Domain.Cache;
using XF.Domain.Dependency;
using XF.Domain.Entities;
using XF.Domain.Exceptions;
using XF.Domain.Extensions;

namespace Hotline.Orders;

public class OrderDomainService : IOrderDomainService, IScopeDependency
{
    private const string OrderNoPrefix = "OrderNo:";
    private readonly IOrderRepository _orderRepository;
    private readonly ITypedCache<CacheOrderNO> _cacheOrderNo;
    private readonly ISessionContext _sessionContext;

    public OrderDomainService(
        IOrderRepository orderRepository,
        ITypedCache<CacheOrderNO> cacheOrderNo,
        ISessionContext sessionContext)
    {
        _orderRepository = orderRepository;
        _cacheOrderNo = cacheOrderNo;
        _sessionContext = sessionContext;
    }

    public async Task<string> AddAsync(Order order, CancellationToken cancellationToken)
    {
        if (order.AcceptType == EAcceptType.Complain && order.OrderComplain == null)
            throw UserFriendlyException.SameMessage("非法投诉参数");
        if (order.AcceptType == EAcceptType.Complain && order.OrderReport == null)
            throw UserFriendlyException.SameMessage("非法举报参数");

        order.Init(_sessionContext.RequiredUserId);
        order.No = GenerateNewOrderNo();

        return await _orderRepository.AddOrderNavAsync(order, cancellationToken);
    }

    /// <summary>
    /// 归档
    /// </summary>
    public async Task FileAsync(string orderId, CancellationToken cancellationToken)
    {
        var order = await _orderRepository.GetAsync(orderId, cancellationToken);
        if (order == null)
            throw UserFriendlyException.SameMessage("无效的工单编号");
        order.Status = EOrderStatus.Filed;
        await _orderRepository.UpdateAsync(order, cancellationToken);
    }

    /// <summary>
    /// 工单办理(每个节点都会触发)
    /// </summary>
    public async Task OrderManageAsync(EOrderStatus status, FlowAssignMode assignMode,
        bool isCountersignEnd, bool isCountersignStart,
        string workflowId, DateTime? currentStepTime, string? CurrentStepName, DateTime expiredTime,
        CancellationToken cancellationToken)
    {
        var order = await _orderRepository.GetAsync(d => d.WorkflowId == workflowId, cancellationToken);
        if (order == null)
            throw new UserFriendlyException($"无效工单流程编号, workflowId: {workflowId}", "无效工单编号");
        if (order.Status is EOrderStatus.Filed) return;

        //更新指派信息
        order.Assign(assignMode.FlowAssignType, assignMode.Handlers);

        //1.如果order未处于会签中，则判断是否发起会签（isstartCountersign） 2.如果处于会签中，则判断会签是否结束（isCountersignEnd）
        if (order.Status is EOrderStatus.Countersigning && isCountersignEnd)
        {
            order.Status = EOrderStatus.WaitForSign;
        }
        else if (order.Status is not EOrderStatus.Countersigning && isCountersignStart)
        {
            order.Status = EOrderStatus.Countersigning;
        }

        order.CurrentStepTime = currentStepTime;
        order.CurrentStepName = CurrentStepName;
        order.ExpiredTime = expiredTime;
        order.Status = status;

        await _orderRepository.UpdateAsync(order, cancellationToken);

        //todo push msg
    }

    #region private

    private string GenerateNewOrderNo()
    {
        var today = DateTime.Today;
        var cacheKey = $"{OrderNoPrefix}{today:yyyyMMdd}";
        var cacheOrderNo = _cacheOrderNo.GetOrAdd(cacheKey, f =>
        {
            var todayOrderCount = _orderRepository.Queryable(true)
                .CountAsync(d => d.CreationTime.Date == today.Date)
                .GetAwaiter()
                .GetResult();
            return new CacheOrderNO { TotalCount = todayOrderCount };
        }, ExpireMode.Absolute, TimeSpan.FromDays(1));
        cacheOrderNo.TotalCount += 1;
        var no = GenerateOrderNo(today, cacheOrderNo.TotalCount);
        _cacheOrderNo.Update(cacheKey, d => cacheOrderNo);
        return no;
    }

    private string GenerateOrderNo(DateTime today, int count)
    {
        return $"{today:yyyyMMdd}{count.ToString("000000")}";
    }

    #endregion
}

public class CacheOrderNO
{
    public int TotalCount { get; set; }
}
